@@ -1,4 +1,5 @@
 from functools import reduce
+from itertools import tee, islice
 
 
 def generator_from_list(data):
@@ -39,9 +40,7 @@ class Stream():
         return self
 
     def take(self, number):
-        results = self.__convert_as_list()
-        updated_value = results[:number]
-        self.data = generator_from_list(updated_value)
+        self.data = islice(self.data, number)
         return self
 
     def distinct(self):
@@ -56,16 +55,15 @@ class Stream():
         return self
 
     def __create_a_copy(self):
-        result = list(self.data)
-        self.data = generator_from_list(result)
-        return result
+        self.data, other = tee(self.data)
+        return other
 
     def __convert_as_list(self):
         return list(self.data)
 
     def stream(self):
         result = self.__create_a_copy()
-        return Stream(generator_from_list(result))
+        return Stream(result)
 
     def length(self):
         results = self.__convert_as_list()
